@@ -11,7 +11,6 @@ use strict;
 use Data::Dumper;
 use Getopt::Long;
 use Pod::Usage;
-use WebService::KvKAPI;
 
 my %options = ();
 
@@ -32,35 +31,44 @@ GetOptions(\%options, qw(
     tradeName=s
     q=s
     rsin=s
+    rsinWorkaround
 ));
 
 if ($options{help}) {
     pod2usage({verbose => 1, exitval => 0});
 }
 
+if (!keys %options) {
+    pod2usage({verbose => 1, exitval => 1});
+}
+
 my $profile_search = delete $options{profile};
 my $api_key        = delete $options{apiKey};
 my $raw            = delete $options{raw};
 
 my $api;
+
 if ($api_key) {
+    use WebService::KvKAPI;
     $api = WebService::KvKAPI->new(
         api_key => $api_key,
+        $options{rsinWorkaround}
+        ? (rsin_workaround => $options{rsinWorkaround})
+        : (),
     );
 }
 else {
     use WebService::KvKAPI::Spoof;
     print "Using spoof mode, no api key given", $/;
+
     $api = WebService::KvKAPI::Spoof->new(
         api_key => 'spoofmode',
+        $options{rsinWorkaround}
+        ? (rsin_workaround => $options{rsinWorkaround})
+        : (),
     );
 }
 
-if (!keys %options) {
-    pod2usage({verbose => 1, exitval => 1});
-}
-
-
 if ($raw) {
     print Dumper $api->api_call($raw, \%options);
 }
@@ -83,7 +91,7 @@ query_kvk.pl - Query the Dutch Chamber of Commerce via the CLI
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 SYNOPSIS
 
